#ifndef STARSInstructionID_h
#define STARSInstructionID_h

#include <stdint.h>
#include <assert.h>
#include <map>

class STARS_Instruction_t;

class STARS_InstructionID_t
{
	public:

		// constructor sets private data
		STARS_InstructionID_t(uintptr_t p_id) : m_id(p_id) { }

		
		/* for ease of porting, we allow casting directly to an EA_t */
		operator const STARS_ea_t () const {return m_id; }

		/* most of the time, though, we should map the ID to an insn */
		const STARS_Instruction_t* GetInstruction() 
		{ 
			const STARS_Instruction_t* ret=id_to_insn_map[m_id]; 
			assert(ret); 
			return ret;
		};
		operator const STARS_Instruction_t* ()  { return GetInstruction(); }


		void AddIDToInsnMapping(const STARS_Instruction_t* to) const { id_to_insn_map[m_id]=to; } 

	protected:

		uintptr_t m_id;
		static std::map<uintptr_t, const STARS_Instruction_t*> id_to_insn_map;

};


inline bool operator==(const STARS_ea_t& lhs, const STARS_InstructionID_t& rhs)
{ 
	return lhs==((STARS_ea_t)rhs);
}

inline bool operator!=(const STARS_ea_t& lhs, const STARS_InstructionID_t& rhs)
{
	return !operator==(lhs,rhs);
}


inline bool operator==(const STARS_InstructionID_t& lhs, const STARS_ea_t& rhs)
{
	return operator==(rhs,lhs);
}

inline bool operator!=(const STARS_InstructionID_t& lhs, const STARS_ea_t& rhs)
{
	return !operator==(rhs,lhs);
}

#endif



