#ifndef STARSOp_h
#define STARSOp_h

#include <stdint.h>
#include <assert.h>
#include <map>

class STARS_op_t
{
	public:
		
		// Constructors
		STARS_op_t() = default;

		// Clone method (deep copy).
		virtual STARSOpndTypePtr clone(void) const = 0;

		// Operators
		virtual bool operator<(const STARS_op_t &rOp) const = 0; // Less-than operator for use in STL ordered containers, e.g. sets.

		// Get methods
		virtual uintptr_t GetAddr(void) const = 0;
		virtual unsigned char GetType(void) const = 0; // Get type o_reg, o_displ, etc.
		virtual uint16 GetReg(void) const = 0;  // Get reg field of operand, whether it is an addressing reg or directly used reg in register operand
		virtual char GetSIB(void) const = 0; // Get x86 SIB byte (dense encoding of base reg, index reg, and scale factor)
		virtual char GetSpecFlag4(void) const = 0; // Get specflag4 byte, used to hold a copy of the auxpref byte in x86-64 programs.
		virtual uintptr_t GetImmedValue(void) const = 0; // Get value field for immediate operands; uint32 for x86-32, uint64 for x86-64

		// Set methods
		virtual void SetSpecFlag4(char value) = 0;
		virtual void SetBitInSpecFlag4(char value) = 0; // OR in the value to set a bit

		// Query methods
		virtual bool IsRegOp(void) const = 0;
		virtual bool IsVoidOp(void) const = 0;
		virtual bool IsMemDisplacementOp(void) const = 0;
		virtual bool IsStaticMemOp(void) const = 0;
		virtual bool IsMemNoDisplacementOp(void) const = 0;
		virtual bool HasSIBByte(void) const = 0;


};

#endif



