#ifndef STARS_IDA_Instruction_h
#define STARS_IDA_Instruction_h

#include <memory>
#include <vector>
#include <pro.h>
#include <ua.hpp>
#include "interfaces/SMPDBInterface.h"
#include "interfaces/idapro/all.h"

using namespace std;

// struct to hold items that mimic the IDA Pro type insn_t.
struct STARS_IDA_insn_t
{

#if 0  // duplicate info

// Current segment base paragraph. Initialized by the kernel.

  ea_t cs;                      // segment base (in paragraphs)


// Virtual address of the instruction (address within the segment)
// Initialized by the kernel.

  ea_t ip;                      // offset in the segment


// Linear address of the instruction.
// Initialized by the kernel.

  ea_t ea;                      // instruction start addresses
#endif

// Internal code of instruction. IDP should define its own instruction
// codes. These codes are usually defined in ins.hpp. The array of instruction
// names and features (ins.cpp) is accessed using this code.

  uint16 itype;                 // instruction code (see ins.hpp)
                                // only for canonical insns (not user defined!):

// Size of instruction in bytes.
// The analyzer should put here the actual size of the instruction.

  uint16 size;                  // instruction size in bytes


// Additional information about the instruction.
// Definitions are processor-dependent.

  union
  {
    uint16 auxpref;             // processor dependent field
    struct
    {
      uchar low;
      uchar high;
    } auxpref_chars;
  };
  char segpref;                 // processor dependent field
  char insnpref;                // processor dependent field

// Information about instruction operands.
  vector<STARSOpndTypePtr> Operands;

  char flags;                   // instruction flags

}; // end of struct STARS_IDA_insn_t

class STARS_IDA_Instruction_t : public STARS_Instruction_t
{
	public:
		
		STARS_IDA_Instruction_t(const STARS_InstructionID_t& p_id) : STARS_Instruction_t(p_id) {};
		STARS_InstructionID_t GetNextInstructionID(void);
		STARS_InstructionID_t GetTargetInstructionID(void);
		bool STARS_GetCmd(void);
		inline uint32 GetFeatures(void) const { return STARSfeatures; };
		inline bool IsRegOpnd(size_t OpndNum) const { return (STARScmd.Operands.at(OpndNum)->GetType() == o_reg); };

		STARSOpndTypePtr GetOpnd(size_t OpndNum) const { return STARScmd.Operands.at(OpndNum); };
		STARSOpndTypePtr MakeVoidOpnd(void) const ;
		STARSOpndTypePtr MakeImmediateOpnd(uintptr_t value) const ;
		STARSOpndTypePtr MakeRegOpnd(uint16 RegNum) const ;
		STARSOpndTypePtr MakeFloatingPointRegOpnd(uint16 RegNum) const ;
		STARSOpndTypePtr MakeMMXRegOpnd(uint16 RegNum) const ;
		STARSOpndTypePtr MakeXMMRegOpnd(uint16 RegNum) const ;
		STARSOpndTypePtr MakeYMMRegOpnd(uint16 RegNum) const ;
		STARSOpndTypePtr MakeMemDisplacementOpnd(uint16 BaseRegNum, uint16 IndexRegNum, uint16_t ScaleFactor, uintptr_t offset) const ;
		STARSOpndTypePtr MakeMemPhraseOpnd(uint16 BaseRegNum, uint16 IndexRegNum, uint16_t ScaleFactor) const ;

	protected:
		struct STARS_IDA_insn_t STARScmd;
		uint32 STARSfeatures;

};

#endif



