#ifndef STARS_IDA_op_h
#define STARS_IDA_op_h

#include <stdint.h>
#include <pro.h>
#include <ua.hpp>

class STARS_IDA_op_t : public STARS_op_t
{
	public:
		
		// Constructors
		STARS_IDA_op_t(op_t IDAOp) : m_Opnd(IDAOp) {};

		// Clone method (deep copy).
		STARSOpndTypePtr clone(void) const;

		// Operators
		bool operator<(const STARS_op_t &rOp) const; // Less-than operator for use in STL ordered containers, e.g. sets.

		// Get methods
		uintptr_t GetAddr(void) const { return m_Opnd.addr; };
		unsigned char GetType(void) const { return m_Opnd.type; };
		uint16 GetReg(void) const { return m_Opnd.reg; };
		char GetSIB(void) const { return m_Opnd.sib; }; // Get x86 SIB byte (dense encoding of base reg, index reg, and scale factor)
		char GetSpecFlag4(void) const {return m_Opnd.specflag4; }; // Get specflag4 byte, used to hold a copy of the auxpref byte in x86-64 programs.
		uintptr_t GetImmedValue(void) const {return m_Opnd.value; }; // Get value field for immediate operands; uint32 for x86-32, uint64 for x86-64

		// Set methods
		void SetSpecFlag4(char value) { m_Opnd.specflag4 = value; };
		void SetBitInSpecFlag4(char value) { m_Opnd.specflag4 |= value; };

		// Query methods
		bool IsRegOp(void) const { return (m_Opnd.type == o_reg); };
		bool IsVoidOp(void) const { return (m_Opnd.type == o_void); };
		bool IsMemDisplacementOp(void) const { return (m_Opnd.type == o_displ); };
		bool IsStaticMemOp(void) const { return (m_Opnd.type == o_mem); };
		bool IsMemNoDisplacementOp(void) const { return (m_Opnd.type == o_phrase); };
		bool HasSIBByte(void) const { return (m_Opnd.hasSIB != 0); };

	protected:
		op_t m_Opnd;
};

#endif



