/*
 * SMPDBInterface.cpp - <see below>.
 *
 * Copyright (c) 2000, 2001, 2010 - University of Virginia 
 *
 * This file is part of the Memory Error Detection System (MEDS) infrastructure.
 * This file may be used and modified for non-commercial purposes as long as 
 * all copyright, permission, and nonwarranty notices are preserved.  
 * Redistribution is prohibited without prior written consent from the University 
 * of Virginia.
 *
 * Please contact the authors for restrictions applying to commercial use.
 *
 * THIS SOURCE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: University of Virginia
 * e-mail: jwd@virginia.com
 * URL   : http://www.cs.virginia.edu/
 *
 * Additional copyrights 2010, 2011 by Zephyr Software LLC
 * e-mail: {clc,jwd}@zephyr-software.com
 * URL   : http://www.zephyr-software.com/
 *
 */

//
// SMPDBInterface.cpp
//
// This module acts as middleware so that STARS can operate using IDA Pro info 
//   or IRDB (Intermediate Representation Database) info for instructions, functions,
//   segments, etc.
//

#include <string>
#include <utility>
#include <list>
#include <set>
#include <vector>
#include <algorithm>

#include <cstring>
#include <cstdlib>

#include <pro.h>
#include <ua.hpp>
#include <assert.h>
#include <ida.hpp>
#include <idp.hpp>
#include <auto.hpp>
#include <bytes.hpp>
#include <funcs.hpp>
#include <intel.hpp>
#include <name.hpp>

#include "SMPDBInterface.h"
#include "SMPStaticAnalyzer.h"
#include "SMPDataFlowAnalysis.h"

#ifdef STARS_IDA_INTERFACE

// Get instruction info by address from IDA Pro.
bool SMPGetCmd(ea_t InstAddr, insn_t &SMPcmd, uint32 &SMPfeatures) {
	bool success = true;

	// Fill cmd structure with disassembly of instr
#if IDA_SDK_VERSION < 600
	int InstrLen = ua_ana0(InstAddr);
#else
	int InstrLen = decode_insn(InstAddr);
#endif
	if (0 >= InstrLen) {
		SMP_msg("ERROR: decode_insn failed at %lx \n", (unsigned long) InstAddr);
		SMPcmd = cmd;
		SMPcmd.size = 0;
		success = false;
		return success;
	}

	// Copy cmd to argument SMPcmd.
	SMPcmd = cmd;
	// Get the canonical features into argument SMPfeatures.
	SMPfeatures = cmd.get_canon_feature();

	for (int i = 0; i < UA_MAXOP; ++i) {
		SMPcmd.Operands[i].specflag4 = 0; 
#ifdef __EA64__
		if (STARS_ISA_Bitwidth == 64) {
			// Copy the cmd.rex prefix into the op_t.specflag4 field for each operand
			//  that has a SIB byte.
			SMPcmd.Operands[i].specflag4 = SMPcmd.rex;
		}
#endif
		// See comments on STARS_VEXPR and STARS_VSIB in SMPDataFlowAnalysis.h.
		//  These bits do not (as of IDA Pro 6.4) conflict with cmd.rex bits.
		if ((cmd.auxpref & aux_vexpr) != 0) {
			SMPcmd.Operands[i].specflag4 |= STARS_VEXPR;
		}

		switch (SMPcmd.itype) {
		    case NN_vgatherdps:
			case NN_vgatherdpd:
			case NN_vgatherqps:
			case NN_vgatherqpd:
			case NN_vpgatherdd:
			case NN_vpgatherdq:
			case NN_vpgatherqd:
			case NN_vpgatherqq:
			  SMPcmd.Operands[i].specflag4 |= STARS_VSIB;
			default:
				;
		}
	}

	return success;
} // end of SMPGetCmd()


#else //STARS_IRDB_INTERFACE

FILE *SMPLogFile;

#endif
